# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-10-24 18:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0003_auto_20191024_1604'),
    ]

    operations = [
        migrations.AlterField(
            model_name='circle',
            name='jabberID',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
    ]
