# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-10-29 13:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0005_auto_20191029_1227'),
    ]

    operations = [
        migrations.AddField(
            model_name='circle',
            name='status',
            field=models.CharField(choices=[('Public', 'Public'), ('Private', 'Private'), ('Archived', 'Archived')], default='Public', max_length=8),
        ),
    ]
