# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-10-24 16:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0002_auto_20191021_1357'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='circlemember',
            options={},
        ),
    ]
