# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-05-12 13:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0017_auto_20200512_1348'),
    ]

    operations = [
        migrations.AlterField(
            model_name='circle',
            name='description',
            field=models.CharField(blank=True, default='', max_length=255, null=True),
        ),
    ]
