# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-05-06 19:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0015_auto_20200505_1733'),
        ('djangoldp_circle', '0014_auto_20200506_1517'),
    ]

    operations = [
    ]
