# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-11-12 12:51
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoldp_circle', '0006_circle_status'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='circlemember',
            unique_together=set([('user', 'circle')]),
        ),
    ]
