# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-04-26 16:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_circle', '0010_auto_20200414_0908'),
    ]

    operations = [
        migrations.AlterField(
            model_name='circle',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='owned_circles', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='circlemember',
            name='circle',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='members', to='djangoldp_circle.Circle'),
            preserve_default=False,
        ),
    ]
